/**
 *
 * @file p_spm.h
 *
 * SParse Matrix package precision dependent header.
 *
 * @copyright 2016-2022 Bordeaux INP, CNRS (LaBRI UMR 5800), Inria,
 *                      Univ. Bordeaux. All rights reserved.
 *
 * @version 1.1.0
 * @author Pierre Ramet
 * @author Mathieu Faverge
 * @author Alban Bellot
 * @author Tony Delarue
 * @date 2021-04-04
 *
 * @generated from C:/M/mingw-w64-pastix/src/pastix-6.2.2/spm/src/z_spm.h, normal z -> p, Fri Feb 24 18:28:28 2023
 *
 **/
#ifndef _p_spm_h_
#define _p_spm_h_

/**
 * Integer routines
 */
void p_spmIntFltSortAsc(    void ** const pbase, const spm_int_t n );
void p_spmIntIntFltSortAsc( void ** const pbase, const spm_int_t n );

/**
 * Conversion routines
 */
int p_spmConvertCSC2CSR( spmatrix_t *spm );
int p_spmConvertCSC2IJV( spmatrix_t *spm );
int p_spmConvertCSR2CSC( spmatrix_t *spm );
int p_spmConvertCSR2IJV( spmatrix_t *spm );
int p_spmConvertIJV2CSC( spmatrix_t *spm );
int p_spmConvertIJV2CSR( spmatrix_t *spm );

int *p_spm2dense( const spmatrix_t *spm );

/**
 * Matrix-Vector and matrix-matrix product routines
 */
int spm_pspmv( spm_trans_t            trans,
               int        alpha,
               const spmatrix_t      *A,
               const int *x,
               spm_int_t              incx,
               int        beta,
               int       *y,
               spm_int_t              incy );
int spm_pspmm( spm_side_t             side,
               spm_trans_t            transA,
               spm_trans_t            transB,
               spm_int_t              K,
               int        alpha,
               const spmatrix_t      *A,
               const int *B,
               spm_int_t              ldb,
               int        beta,
               int       *C,
               spm_int_t              ldc );

/**
 * Norm computation routines
 */
int p_spmNorm( spm_normtype_t ntype, const spmatrix_t *spm );

/**
 * Extra routines
 */
void      p_spmSort( spmatrix_t *spm );
spm_int_t p_spmMergeDuplicate( spmatrix_t *spm );

int              p_spmGenRHS( spm_rhstype_t type, int nrhs, const spmatrix_t *spm, void *x, int ldx, void *b, int ldb );
int              p_spmGenMat( spm_rhstype_t type, int nrhs, const spmatrix_t *spm, void *alpha, unsigned long long int seed, void *A, int lda );
int              p_spmCheckAxb( spm_fixdbl_t eps, int nrhs, const spmatrix_t *spm, void *x0, int ldx0, void *b, int ldb, const void *x, int ldx );

int *p_spmGatherRHS      ( int nrhs, const spmatrix_t *spm, const int *x,     spm_int_t ldx, int root );
void             p_spmReduceRHS      ( int nrhs, const spmatrix_t *spm,       int *bglob, spm_int_t ldbg, int *bloc, spm_int_t ldbl );
void             p_spmExtractLocalRHS( int nrhs, const spmatrix_t *spm, const int *bglob, spm_int_t ldbg, int *bloc, spm_int_t ldbl );

/**
 * Output routines
 */
void p_spmDensePrint( FILE *f, spm_int_t m, spm_int_t n, const int *A, spm_int_t lda );
void p_spmPrint( FILE *f, const spmatrix_t *spm );
void p_spmPrintRHS( FILE *f, const spmatrix_t *spm, int nrhs, const void *x, spm_int_t ldx );

void p_spmExpand( const spmatrix_t *spm_in, spmatrix_t *spm_out );
void p_spmDofExtend( spmatrix_t *spm );
void p_spmScal( const int alpha, spmatrix_t *spm );

/**
 * Subroutines for random vector generation to be used in testings
 */
int p_spmRhsGenRndShm( const spmatrix_t *spm, int scale,
                       spm_int_t n, int *A, spm_int_t lda,
                       int shift, unsigned long long int seed );
int p_spmRhsGenRndDist( const spmatrix_t *spm, int scale,
                        spm_int_t n, int *A, spm_int_t lda,
                        int shift, unsigned long long int seed );

#endif /* _p_spm_h_ */
